//----------------------------------------------------------------------------------
//	FILE:			Inversor_Multinivel-Main.C
//
//	Descripcion:	Control en lazo Abierto de los 12 transistores de un inversor multi-mivel
//
//	Version: 		1.0
//
//  Tarjeta:  		TMS320F28069
//
//	Tipo: 			Docking-Sth USB_EMU
//
//----------------------------------------------------------------------------------
//
// ALGUNAS NOTAS SOBRE ESTE PROYECTO

// Todo este proyecyto esta formado por diversos ficheros, los mas importantes son:
//	 "Inversor_Multinivel-Main.C" - este archivo
//		- Inicializacion de las configuraciones, configuraciones externas
//		- Gestion de la aplicacion
//		- Lazos de control y planificacion de tareas
//	 "Inversor_Multinivel-DevInit_F28xxx.C
//		- Inicializacion del despositivo, ej. Reloj, PLL, WD, mapa GPIO
//		- Habilitacion de diversos relojes


//----------------------------------------------------------------------------------
// Este programa generar 12 seales PWM configuradas para controlar en lazo abierto
// un inversor multi-nivel trifasico. Solo se podra elegir la frecuencia de trabajo
//----------------------------------------------------------------------------------

#include "PeripheralHeaderIncludes.h"
#include "F2806x_EPwm_defines.h" 	    // Definiciones para incializaciones
																		 

//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
// FUNCIONES
//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

// -------------------------------- GENERALES --------------------------------------
void DeviceInit(void);
void SCIA_Init();
void SerialHostComms();
void InitFlash();
void MemCopy();

// --------------------------------- USUARIO ----------------------------------------



//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
// DECLARACION DE VARIABLES
//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

// -------------------------------- GENERALES --------------------------------------
int16	SerialCommsTimer;
int16	CommsOKflg;
// Usadas para ejecuciones en segundo plano con memoria flash, y ISR con memoria RAM
extern Uint16 *RamfuncsLoadStart, *RamfuncsLoadEnd, *RamfuncsRunStart;


// --------------------------------- USUARIO ----------------------------------------
// Dado el tiempo t en ns, el dead time sigue la ecuacion: RED = FED = 0.06*t
// Esta ecuacion es valida cuando el PLL esta configurado a 0xC (60MHz)
Uint16 RED_delay=6; // 100ns
Uint16 FED_delay=6; // 100ns


//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
// VARIABLES PARA SOPORTE GUI
//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

// -------------------------------- FRAMEWORK --------------------------------------

// Establece una cantidad limite para los controles externos de GUI
// Incrementar si es necesario
int16 	*varSetTxtList[8];				//8 textbox controlled variables
int16 	*varSetBtnList[8];				//8 button controlled variables
int16 	*varSetSldrList[8];				//8 slider controlled variables
int16 	*varGetList[8];					//8 variables sendable to GUI
int16 	*arrayGetList[8];				//8 arrays sendable to GUI



//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
// CODIGO PRINCIPAL
//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

void main(void)
{
//=================================================================================
//	Inicializacin del dispositivo
//=================================================================================

	DeviceInit();	// Inizializacion y mapa de entradas/salidas
	SCIA_Init();  	// Inicializa periferico Serial Comms A

// Solo se ejecutar al seleccionar ejecucin con memoria FLASH
#ifdef FLASH		
// Copy time critical code and Flash setup code to RAM
// The  RamfuncsLoadStart, RamfuncsLoadEnd, and RamfuncsRunStart
// symbols are created by the linker. Refer to the linker files. 
	MemCopy(&RamfuncsLoadStart, &RamfuncsLoadEnd, &RamfuncsRunStart);

// Call Flash Initialization to setup flash waitstates
// This function must reside in RAM
	InitFlash();	// Call the flash wrapper init function
#endif //(FLASH)


// --------------------------------- USUARIO ----------------------------------------
// Se podran modificar estas variables en cada ejecucion en funcion de las necesidades

// El periodo se selecciona en funcin de la frecuencia deseada siguiendo esta ecuacion
// period=(60*10^6)/2*frecuencia, estando la frecuencia deseada en Hz
// Ecuacin valida cuando el PLL esta configurado a 60MHz 0xC
#define period 30000	// 1kHz
//Frecuencia max - 50 KHz (limitado por driver), Frecuencia min - 230 Hz (limitado por DSP)


//=================================================================================
//	Inicializacin de las seales PWM
//=================================================================================


	//=================================
	//	EPwm1.A es transistor S1 de la rama A (PIN 00)
	//  EPwm1.B es transistor S11 de la rama A (PIN 01)
	//=================================

    //Registros TIME-BASE
    EPwm1Regs.TBPRD = period/2;						//Periodo de la seal
    EPwm1Regs.TBPHS.all = 0;						//Inicializacion del registro Time-Base Phase
    EPwm1Regs.TBPHS.half.TBPHS = 0;					//Inicializacion del registro Time-Base Phase, angulo de desfase
    EPwm1Regs.TBCTR = 0;							//Inicializacion del registro Time-Base Counter
    EPwm1Regs.TBCTL.bit.PRDLD = TB_IMMEDIATE;		//Inicio Inmediato, lecturas directas al registro activo
    EPwm1Regs.TBCTL.bit.CTRMODE = TB_COUNT_UPDOWN;	//Modo ascendente-descendente, triangular
   	EPwm1Regs.TBCTL.bit.PHSEN = TB_DISABLE;			//Deshabilitada sincronizacin
   	EPwm1Regs.TBCTL.bit.SYNCOSEL = TB_CTR_ZERO;		//Envia seal de sincronismo cuando pase por CTR=0

   	//Registro Counter-Compare
    EPwm1Regs.CMPCTL.bit.SHDWAMODE = CC_SHADOW;		//Modo Shadown, operando como un doble buffer
    EPwm1Regs.CMPCTL.bit.SHDWBMODE = CC_SHADOW;		//Modo Shadown, operando como un doble buffer
    EPwm1Regs.CMPCTL.bit.LOADAMODE = CC_CTR_ZERO;	//Carga en CTR=Zero
    EPwm1Regs.CMPCTL.bit.LOADBMODE = CC_CTR_ZERO;	//Carga en CTR=Zero

    //Establecemos ciclo de servicio
    EPwm1Regs.CMPA.half.CMPA = period/4;    		//25%
    EPwm1Regs.CMPB = period/4;

    //Aciones PWM
    EPwm1Regs.AQCTLA.bit.ZRO = AQ_SET;				//PWM=1
    EPwm1Regs.AQCTLA.bit.CAU = AQ_CLEAR;			//PWM=0

    EPwm1Regs.AQCTLB.bit.ZRO = AQ_CLEAR;			//PWM=0
    EPwm1Regs.AQCTLB.bit.CBU = AQ_SET;				//PWM=1

    //Configuracion de tiempos muertos
	EPwm1Regs.DBCTL.bit.IN_MODE = DBA_ALL;			//Tiempo muerto de subida y bajada para seales A y B
	EPwm1Regs.DBCTL.bit.OUT_MODE = DB_FULL_ENABLE;	//Tiempo muerto de subida y bajada para seales A y B
	EPwm1Regs.DBCTL.bit.POLSEL = DB_ACTV_HIC;		//Activamos High Complementary (AHC)
	EPwm1Regs.DBRED = RED_delay;					//Tiempo muerto en flanco de subida
	EPwm1Regs.DBFED = FED_delay;					//Tiempo muerto en flanco de bajada


	//=================================
	//	EPwm2.A es transistor S22 de la rama A (PIN 02)
	//  EPwm2.B es transistor S2 de la rama A (PIN 03)
	//=================================

    //Registros TIME-BASE
    EPwm2Regs.TBPRD = period/2;
    EPwm2Regs.TBPHS.all = 0;
    EPwm2Regs.TBPHS.half.TBPHS = period/2;			//Angulo de desfase 180 con respecto PWM1
    EPwm2Regs.TBCTR = 0;
    EPwm2Regs.TBCTL.bit.PRDLD = TB_IMMEDIATE;
    EPwm2Regs.TBCTL.bit.CTRMODE = TB_COUNT_UPDOWN;	//Modo ascendente-descendente, triangular
    EPwm2Regs.TBCTL.bit.PHSDIR = TB_DOWN;			//Cuando sincroniza, empieza la seal triangular descendente
   	EPwm2Regs.TBCTL.bit.PHSEN = TB_ENABLE;	   		//Habilitamos sincronizacion
   	EPwm2Regs.TBCTL.bit.SYNCOSEL = TB_SYNC_IN;		//Seal de salida de sincronismo es la misma que la de entrada
   													//Se realimenta la seal CTR=0 de la rama PWM1 como seal de sincronismo

   	//Registro Counter-Compare
    EPwm2Regs.CMPCTL.bit.SHDWAMODE = CC_SHADOW;
    EPwm2Regs.CMPCTL.bit.SHDWBMODE = CC_SHADOW;
    EPwm2Regs.CMPCTL.bit.LOADAMODE = CC_CTR_ZERO;
    EPwm2Regs.CMPCTL.bit.LOADBMODE = CC_CTR_ZERO;

    //Establecemos ciclo de servicio
    EPwm2Regs.CMPA.half.CMPA = period/4;
    EPwm2Regs.CMPB = period/4;

    //Aciones PWM
    EPwm2Regs.AQCTLA.bit.ZRO = AQ_CLEAR;
    EPwm2Regs.AQCTLA.bit.CAU = AQ_SET;

    EPwm2Regs.AQCTLB.bit.ZRO = AQ_SET;
    EPwm2Regs.AQCTLB.bit.CBU = AQ_CLEAR;

    //Configuracion de tiempos muertos
	EPwm2Regs.DBCTL.bit.IN_MODE = DBA_ALL;
	EPwm2Regs.DBCTL.bit.OUT_MODE = DB_FULL_ENABLE;
	EPwm2Regs.DBCTL.bit.POLSEL = DB_ACTV_HIC;
	EPwm2Regs.DBRED = RED_delay;
	EPwm2Regs.DBFED = FED_delay;



	//=================================
	//	EPwm3.A es transistor S1 de la rama B (PIN 04)
	//  EPwm3.B es transistor S11 de la rama B (PIN 05)
	//=================================

    //Registros TIME-BASE
    EPwm3Regs.TBPRD = period/2;
    EPwm3Regs.TBPHS.all = 0;
    EPwm3Regs.TBPHS.half.TBPHS = period/3;			//Angulo de desfase 120 con respecto PWM1
    EPwm3Regs.TBCTR = 0;
    EPwm3Regs.TBCTL.bit.PRDLD = TB_IMMEDIATE;
    EPwm3Regs.TBCTL.bit.CTRMODE = TB_COUNT_UPDOWN;
    EPwm3Regs.TBCTL.bit.PHSDIR = TB_DOWN;			//Cuando sincroniza, empieza la seal triangular descendente
   	EPwm3Regs.TBCTL.bit.PHSEN = TB_ENABLE;
   	EPwm3Regs.TBCTL.bit.SYNCOSEL = TB_CTR_ZERO;		//Envia seal de sincronismo cuando pase por CTR=0
   													//Las posteriores seales sincronizan siguiendo como referencia a PWM3

   	//Registro Counter-Compare
    EPwm3Regs.CMPCTL.bit.SHDWAMODE = CC_SHADOW;
    EPwm3Regs.CMPCTL.bit.SHDWBMODE = CC_SHADOW;
    EPwm3Regs.CMPCTL.bit.LOADAMODE = CC_CTR_ZERO;
    EPwm3Regs.CMPCTL.bit.LOADBMODE = CC_CTR_ZERO;

    //Establecemos ciclo de servicio
    EPwm3Regs.CMPA.half.CMPA = period/4;
    EPwm3Regs.CMPB = period/4;

    //Aciones PWM
    EPwm3Regs.AQCTLA.bit.ZRO = AQ_SET;
    EPwm3Regs.AQCTLA.bit.CAU = AQ_CLEAR;

    EPwm3Regs.AQCTLB.bit.ZRO = AQ_CLEAR;
    EPwm3Regs.AQCTLB.bit.CBU = AQ_SET;

    //Configuracion de tiempos muertos
	EPwm3Regs.DBCTL.bit.IN_MODE = DBA_ALL;
	EPwm3Regs.DBCTL.bit.OUT_MODE = DB_FULL_ENABLE;
	EPwm3Regs.DBCTL.bit.POLSEL = DB_ACTV_HIC;
	EPwm3Regs.DBRED = RED_delay;
	EPwm3Regs.DBFED = FED_delay;


	//=================================
	//	EPwm4.A es transistor S22 de la rama B (PIN 06)
	//  EPwm4.B es transistor S2 de la rama B (PIN 07)
	//=================================

    //Registros TIME-BASE
    EPwm4Regs.TBPRD = period/2;
    EPwm4Regs.TBPHS.half.TBPHS = period/2;			//Angulo de desfase 180 con respecto PWM3
    EPwm4Regs.TBCTR = 0;
    EPwm4Regs.TBCTL.bit.PRDLD = TB_SHADOW;
    EPwm4Regs.TBCTL.bit.CTRMODE = TB_COUNT_UPDOWN;
    EPwm4Regs.TBCTL.bit.PHSDIR = TB_DOWN;
   	EPwm4Regs.TBCTL.bit.PHSEN = TB_ENABLE;
   	EPwm4Regs.TBCTL.bit.SYNCOSEL = TB_SYNC_IN;		//Seal de salida de sincronismo es la misma que la de entrada
													//Se realimenta la seal CTR=0 de la rama PWM3 como seal de sincronismo

   	//Registro Counter-Compare
    EPwm4Regs.CMPCTL.bit.SHDWAMODE = CC_SHADOW;
    EPwm4Regs.CMPCTL.bit.SHDWBMODE = CC_SHADOW;
    EPwm4Regs.CMPCTL.bit.LOADAMODE = CC_CTR_ZERO;
    EPwm4Regs.CMPCTL.bit.LOADBMODE = CC_CTR_ZERO;

    //Establecemos ciclo de servicio
    EPwm4Regs.CMPA.half.CMPA = period/4;
    EPwm4Regs.CMPB = period/4;

    //Aciones PWM
    EPwm4Regs.AQCTLA.bit.ZRO = AQ_CLEAR;
    EPwm4Regs.AQCTLA.bit.CAU = AQ_SET;

    EPwm4Regs.AQCTLB.bit.ZRO = AQ_SET;
    EPwm4Regs.AQCTLB.bit.CBU = AQ_CLEAR;

    //Configuracion de tiempos muertos
	EPwm4Regs.DBCTL.bit.IN_MODE = DBA_ALL;
	EPwm4Regs.DBCTL.bit.OUT_MODE = DB_FULL_ENABLE;
	EPwm4Regs.DBCTL.bit.POLSEL = DB_ACTV_HIC;
	EPwm4Regs.DBRED = RED_delay;
	EPwm4Regs.DBFED = FED_delay;



	//=================================
	//	EPwm5A es transistor S1 de la rama C (PIN 8)
	//  EPwm5B es transistor S11 de la rama C (PIN 9)
	//=================================

    //Registros TIME-BASE
    EPwm5Regs.TBPRD = period/2;
    EPwm5Regs.TBPHS.half.TBPHS = period/3;			//Angulo de desfase 120 con respecto PWM3
    EPwm5Regs.TBCTR = 0;
    EPwm5Regs.TBCTL.bit.PRDLD = TB_SHADOW;
    EPwm5Regs.TBCTL.bit.CTRMODE = TB_COUNT_UPDOWN;
    EPwm5Regs.TBCTL.bit.PHSDIR = TB_DOWN;
   	EPwm5Regs.TBCTL.bit.PHSEN = TB_ENABLE;
   	EPwm5Regs.TBCTL.bit.SYNCOSEL = TB_CTR_ZERO;		//Envia seal de sincronismo cuando pase por CTR=0
   													//Las posteriores seales sincronizan siguiendo como referencia a PWM5

   	//Registro Counter-Compare
    EPwm5Regs.CMPCTL.bit.SHDWAMODE = CC_SHADOW;
    EPwm5Regs.CMPCTL.bit.SHDWBMODE = CC_SHADOW;
    EPwm5Regs.CMPCTL.bit.LOADAMODE = CC_CTR_ZERO;
    EPwm5Regs.CMPCTL.bit.LOADBMODE = CC_CTR_ZERO;

    //Establecemos ciclo de servicio
    EPwm5Regs.CMPA.half.CMPA = period/4;
    EPwm5Regs.CMPB = period/4;

    //Aciones PWM
    EPwm5Regs.AQCTLA.bit.ZRO = AQ_SET;
    EPwm5Regs.AQCTLA.bit.CAU = AQ_CLEAR;

    EPwm5Regs.AQCTLB.bit.ZRO = AQ_CLEAR;
    EPwm5Regs.AQCTLB.bit.CBU = AQ_SET;

    //Configuracion de tiempos muertos
	EPwm5Regs.DBCTL.bit.IN_MODE = DBA_ALL;
	EPwm5Regs.DBCTL.bit.OUT_MODE = DB_FULL_ENABLE;
	EPwm5Regs.DBCTL.bit.POLSEL = DB_ACTV_HIC;
	EPwm5Regs.DBRED = RED_delay;
	EPwm5Regs.DBFED = FED_delay;



	//=================================
	//	EPwm6.A es transistor S22 de la rama C (PIN 10)
	//  EPwm6.B es transistor S2 de la rama C (PIN 11)
	//=================================

    //Registros TIME-BASE
    EPwm6Regs.TBPRD = period/2;
    EPwm6Regs.TBPHS.half.TBPHS = period/2;			//Angulo de desfase 120 con respecto PWM3
    EPwm6Regs.TBCTR = 0;
    EPwm6Regs.TBCTL.bit.PRDLD = TB_SHADOW;
    EPwm6Regs.TBCTL.bit.CTRMODE = TB_COUNT_UPDOWN;
    EPwm6Regs.TBCTL.bit.PHSDIR = TB_DOWN;
   	EPwm6Regs.TBCTL.bit.PHSEN = TB_ENABLE;
   	EPwm6Regs.TBCTL.bit.SYNCOSEL = TB_SYNC_IN;		//Seal de salida de sincronismo es la misma que la de entrada
													//Se realimenta la seal CTR=0 de la rama PWM3 como seal de sincronismo
   													//Usariamos esta seal de sincronismo para una posible futura seal PWM7

   	//Registro Counter-Compare
    EPwm6Regs.CMPCTL.bit.SHDWAMODE = CC_SHADOW;
    EPwm6Regs.CMPCTL.bit.SHDWBMODE = CC_SHADOW;
    EPwm6Regs.CMPCTL.bit.LOADAMODE = CC_CTR_ZERO;
    EPwm6Regs.CMPCTL.bit.LOADBMODE = CC_CTR_ZERO;

    //Establecemos ciclo de servicio
    EPwm6Regs.CMPA.half.CMPA = period/4;
    EPwm6Regs.CMPB = period/4;

    //Aciones PWM
    EPwm6Regs.AQCTLA.bit.ZRO = AQ_CLEAR;
    EPwm6Regs.AQCTLA.bit.CAU = AQ_SET;

    EPwm6Regs.AQCTLB.bit.ZRO = AQ_SET;
    EPwm6Regs.AQCTLB.bit.CBU = AQ_CLEAR;

    //Configuracion de tiempos muertos
	EPwm6Regs.DBCTL.bit.IN_MODE = DBA_ALL;
	EPwm6Regs.DBCTL.bit.OUT_MODE = DB_FULL_ENABLE;
	EPwm6Regs.DBCTL.bit.POLSEL = DB_ACTV_HIC;
	EPwm6Regs.DBRED = RED_delay;
	EPwm6Regs.DBFED = FED_delay;



	//=================================================================================
	//	LAZO INFINITO DE EJECUCION
	//================================================================================
	for(;;)
	{
		//En este programa no es necesaria ninguna accion
	}

//FIN DEL CODIGO
}
